<h1>Main page - from template.</h1>
<ul>
   <li><a href="/foo">foo</a> has template - has route = served from route</li>
   <li><a href="/bar">bar</a> has template - no route  = served from template</li>
   <li><a href="/qux">qux</a> no template  - has route = served from route</li>
   <li><a href="/xyz">xyz</a> no template  - no route  = 404</li>
   <li><a href="/cakes/carrot">cakes/carrot</a> template from subirectory </li>
   <li><a href="/cakes/">cakes/</a> root of subirectory from index.html</li>
   <li><a href="/cakes/index.html">/cakes/index.html</a> index.html in subdirectory redirected</li>
   <li><a href="/index.html">/index.html</a> /index.html redirected to /</li>
   <li><a href="/index">/index</a> /index redirected to /</li>
   <li><a href="/robots.txt">robots.txt</a> served from static</li>
   <li><a href="/sitemap.xml">sitemap.xml</a> generated on the fly</li>
</ul>
